import ActionCommand
import Robot

class FSM:
    def __init__(self):
        self.states = {}
        self.history = [{}, {}, {}]
        self.activeState = None
        self.stack = []

    def getStates(self, status = 1):
        return [state.name for state in self.states.values() if state.status == status]
	  
    def activate(self, name):
        self.activeState = self.states[name]
        self.activeState.onActivate()
	  
    def deactivate(self, name):
        self.states[name].onDeactivate()
        self.activeState = None
	
    def add(self, state):
        if state.name in self.states.keys():
            raise "ERROR: state already exists: '" + state.name + "'"
        self.states[state.name] = state
        state.FSM = self
        state.setup()
	
    def reset(self):
        self.states = {}
	
    def step(self):
        for name in self.stack:  
            if self.states[name].interrupt():
                print "interrupted"
                self.activeState.onDeactivate()
                while name != self.activeState.name:
                    self.activeState = self.states[self.stack.pop()]
                break
	    
        self.activeState.step()


class State:
    def __init__(self, name = ''):
        self.behaviors = {}
        self.debug = 0
        self.type = self.__class__.__name__
        self.name = name or self.type
   
    def setup(self):
        pass # normally will overload
		
    def step(self):
        pass # normally will overload
	
    def onActivate(self):
        pass # normally will overload

    def onDeactivate(self):
        pass # normally will overload
        
    def addBehavior(self, behavior):
        if behavior.name in self.behaviors.keys():
            raise "ERROR: behavior already exists: '" + behavior.name + "'"
        self.behaviors[behavior.name] = behavior
        behavior.setup()
        
    def removeBehavior(self, name):
        if name not in self.behaviors.keys():
            raise "ERROR: behavior does not exists: '" + name + "'"
        del self.behaviors[name]
        
    def execute(self, name):
        if name not in self.behaviors.keys():
            raise "ERROR: behavior does not exists: '" + name + "'"
        self.behaviors[name].step()
   
    def getState(self, statename):
        if statename in self.FSM.states.keys():
            return self.FSM.states[statename]
        else:
            raise "ERROR: no such statename"
             
    def getBehavior(self, behaviorname):
        if behaviorname in self.behaviors.keys():
            return self.behaviors[behaviorname]
        else:
            raise "ERROR: no such behaviorname"

    def goto(self, state):
        if self.debug:
            print "Leaving state '%s'; going to state '%s'..." % (self.name, state)
		
        self.onDeactivate()
        self.FSM.activate(state)

    def push(self, statename = None):
        if statename == None:
            statename = self.name
        if statename not in self.FSM.states:
            raise AttributeError, "push: not a valid state name '%s'" % statename
        self.FSM.stack.append( statename )

    def pop(self):
        if len(self.FSM.stack) > 0:
            returnState = self.FSM.stack.pop()
            self.goto(returnState)
        else:
            raise AttributeError, "pop without a push in state '%s'" % self.name
    
    def interrupt(self):
        return False


class Behavior:
    def __init__(self, name = ''):
        self.debug = 0
        self.type = self.__class__.__name__
        self.name = name or self.type

    def setup(self):
        pass # normally will overload
		
    def step(self):
        pass # normally will overload
	
    def onActivate(self):
        pass # normally will overload

    def onDeactivate(self):
        pass # normally will overload




		 
