import Robot
import Info
import ActionCommand
import GameController
from FSM import FSM

from match.Match import Match
from match.Initial import Initial
from match.Ready import Ready
from match.Set import Set
from match.Playing import Playing
from match.Penalised import Penalised
from match.Finished import Finished

from match.Striker import Striker
from match.Goalie import Goalie
from match.Support import Support

from match.KickBall import KickBall
from match.AlignKick import AlignKick
from match.Dribble import Dribble
from match.AlignGoal import AlignGoal
from match.GoToBall import GoToBall
from match.FindBall import FindBall

from match.FindGoal import FindGoal
from match.GoToGoal import GoToGoal
from match.TrackBall import TrackBall
from match.SupportFindBall import SupportFindBall

from match.GoalieTrackBall import GoalieTrackBall
from match.GoalieAlignClear import GoalieAlignClear
from match.GoalieClear import GoalieClear

from match.TestSkill import TestSkill
from match.Calibration import Calibration
from match.MotionTest import MotionTest
from match.OdometryTest import OdometryTest

class MatchSkill:
    
    def __init__(self):
        # Create finite state machine
        self.FSM = FSM()
        
        self.FSM.add(Match())
        self.FSM.add(Initial())
        self.FSM.add(Ready())
        self.FSM.add(Set())
        self.FSM.add(Playing())
        self.FSM.add(Penalised())
        self.FSM.add(Finished())
        
        self.FSM.add(Striker())
        self.FSM.add(Goalie())
        self.FSM.add(Support())

        self.FSM.add(KickBall())
        self.FSM.add(AlignKick())
        self.FSM.add(Dribble())
        self.FSM.add(AlignGoal())
        self.FSM.add(GoToBall())
        self.FSM.add(FindBall())

        self.FSM.add(FindGoal())
        self.FSM.add(GoToGoal())
        self.FSM.add(TrackBall())
        self.FSM.add(SupportFindBall())

        self.FSM.add(GoalieTrackBall())
        self.FSM.add(GoalieAlignClear())
        self.FSM.add(GoalieClear())
        
        self.FSM.add(TestSkill())
        self.FSM.add(Calibration())
        self.FSM.add(MotionTest())
        self.FSM.add(OdometryTest())

        self.FSM.activate('Match')
        
        
    def execute(self):        
        Info.body().actionType(ActionCommand.Body.STAND)
        Info.head().yaw(0.0)
        Info.head().yawSpeed(1.0)
        Info.head().pitch(0.0)
        Info.head().pitchSpeed(1.0)
    
        if Info.getTeamColor() == Info.BLUE:
            Info.led().leftFoot(0,0,1)
        elif Info.getTeamColor() == Info.RED:
            Info.led().leftFoot(1,0,0)
            
        #if Robot.gcData()['secondaryState'] == GameController.NORMAL:
        #    Info.led().rightFoot(1,1,1)
        #elif Robot.gcData()['secondaryState'] == GameController.PENALTYSHOOT:
        #    Info.led().rightFoot(0,0,0)
            
        if Robot.gcData()['state'] == GameController.INITIAL:
            Info.led().chestButton(0,0,0)
        elif Robot.gcData()['state'] == GameController.READY:
            Info.led().chestButton(0,0,1)
        elif Robot.gcData()['state'] == GameController.SET:
            Info.led().chestButton(1,1,0)
        elif Robot.gcData()['state'] == GameController.PLAYING:
            Info.led().chestButton(0,1,0)
        elif Robot.gcData()['state'] == GameController.FINISHED:
            Info.led().chestButton(0,0,0)
        elif Robot.gcData()['state'] == GameController.INVALID:
            Info.led().chestButton(1,0,1)
        elif Robot.gcData()['state'] == GameController.PENALISED:
            Info.led().chestButton(1,0,0)
    
        self.FSM.step()




