import ActionCommand
import Joints
import Robot
import WhichPosts
import WhichCamera
import GameController
import time
import math
import Info
import Util
from FSM import Behavior
from math import degrees, radians, atan2

class AlignGoalSkill(Behavior):
    SHUFFLE_FORWARD = 0
    SHUFFLE_LEFT = 1000
    SHUFFLE_TURN = math.radians(12)

    state = 0    
    globalHeading = 0.0
    turnDirection = 1
    initialScanComplete = False
    alignFinished = False

    def setup(self):
        self.alignFinished = False
        self.state = 0
        self.waypoints = []
        self.waypoints.append((0.0, radians(40.0), 0.2, WhichCamera.TOP_CAMERA))    #left
        self.waypoints.append((0.0, radians(-40.0), 0.2, WhichCamera.TOP_CAMERA))    #right
        self.waypoints.append((0.0, 0.0, 0.2, WhichCamera.TOP_CAMERA))    #center
        #Assuming all goals are yellow
        self.goalEither = 8
        self.goalLeft = 5
        self.goalRight = 6
        self.goalBoth = 7

    def step(self):
        post = Robot.postInfo()['which']
        leftHeading = Robot.postInfo()['pos'][0]['heading']
        rightHeading = Robot.postInfo()['pos'][1]['heading']
        self.globalHeading = Robot.position()['theta']
        self.globalY = Robot.position()['y']
        
        #Initial case: ball and robot are deep on friendly side of the field - clear the ball with less accuracy and without a dependency on seeing enemy goal posts
        if self.globalY < -1000:
            print "CLEARING"
            #Robot is within a 30 degree sector facing the enemy goal
            if self.globalHeading >= 1.309 and self.globalHeading <= 2.007:
                self.initialScanComplete = False
                self.alignFinished = True
            #Turn until the 30 degree sector is reached
            else:
                self.setDirection()
                Util.move(self.SHUFFLE_FORWARD, -self.turnDirection*self.SHUFFLE_LEFT, self.turnDirection*self.SHUFFLE_TURN)
        
        #Optimal case: both goal posts seen:
        elif post == self.goalBoth and self.globalHeading <= math.pi:
            if leftHeading > 0.0 and rightHeading < 0.0:
                self.initialScanComplete = False
                self.alignFinished = True
            elif leftHeading <= 0.0:
                Util.move(self.SHUFFLE_FORWARD, self.SHUFFLE_LEFT, -self.SHUFFLE_TURN)
            else:
                Util.move(self.SHUFFLE_FORWARD, -self.SHUFFLE_LEFT, self.SHUFFLE_TURN)
                
        #Execute initial scan to determine if robot is facing the middle of the enemy goal
        elif not self.initialScanComplete:
            Util.move(0, 0, 0.0)
            if self.globalHeading > math.pi:
                self.initialScanComplete = True
                self.setDirection()
                self.state = 2
            else:
                Util.headScan(self.waypoints, self.state)
                if self.state == 0 and ((post == self.goalLeft or post == self.goalEither) and self.globalHeading <= math.pi) and leftHeading > 0.0:
                    self.state = 1 - self.state
                elif self.state == 1 and ((post == self.goalRight or post == self.goalEither) and self.globalHeading <= math.pi) and leftHeading < 0.0:
                    self.initialScanComplete = False
                    self.alignFinished = True
                else:
                    realPitch = Robot.sensorValues()['joints']['angles'][Joints.HeadPitch]
                    realYaw = Robot.sensorValues()['joints']['angles'][Joints.HeadYaw]
                    if abs(realPitch - Info.head()._pitch) < radians(10) and abs(realYaw - Info.head()._yaw) < radians(10):
                        self.initialScanComplete = True
                        self.setDirection()
                        self.state = 2
         
        #Shuffle to align on the goal
        else:
            Util.headScan(self.waypoints, self.state)
            Util.move(self.SHUFFLE_FORWARD, -self.turnDirection*self.SHUFFLE_LEFT, self.turnDirection*self.SHUFFLE_TURN)
            if (self.turnDirection == 1 and ((post == self.goalRight or post == self.goalEither) and self.globalHeading <= math.pi) and rightHeading < -0.2) or (self.turnDirection == -1 and ((post == self.goalLeft or post == self.goalEither) and self.globalHeading <= math.pi) and leftHeading > 0.2):
                self.initialScanComplete = False
                self.alignFinished = True
    
    def setDirection(self):
        if self.globalHeading <= math.pi/2.0 or self.globalHeading > 3.0*math.pi/2.0:
            self.turnDirection = 1
        elif self.globalHeading > math.pi/2.0 and self.globalHeading <= 3.0*math.pi/2.0:
            self.turnDirection = -1

