import ActionCommand
import Joints
import Robot
import WhichPosts
import WhichCamera
import GameController
import time
import math
import Info
import Util
from FSM import Behavior
from math import degrees, radians, atan2

class AlignGoalSkill(Behavior):
    SHUFFLE_FORWARD = 0
    SHUFFLE_LEFT = 1000
    SHUFFLE_TURN = math.radians(12)

    state = 0    
    globalHeading = 0.0
    turnDirection = 1
    initialScanComplete = False
    alignFinished = False

    def setup(self):
        self.alignFinished = False
        self.state = 0
        self.waypoints = []
        self.waypoints.append((0.0, radians(40.0), 0.2, WhichCamera.TOP_CAMERA))    #left
        self.waypoints.append((0.0, radians(-40.0), 0.2, WhichCamera.TOP_CAMERA))    #right
        self.waypoints.append((0.0, 0.0, 0.2, WhichCamera.TOP_CAMERA))    #center
        if Info.getTeamColor() == Info.BLUE:
            self.enemyGoalEither = 8    #Yellow Goal
            self.enemyGoalLeft = 5
            self.enemyGoalRight = 6
            self.enemyGoalBoth = 7
        elif Info.getTeamColor() == Info.RED:
            self.enemyGoalEither = 4    #Blue Goal
            self.enemyGoalLeft = 1
            self.enemyGoalRight = 2
            self.enemyGoalBoth = 3

    def step(self):
        post = Robot.postInfo()['which']
        leftHeading = Robot.postInfo()['pos'][0]['heading']
        rightHeading = Robot.postInfo()['pos'][1]['heading']
        self.globalHeading = Robot.position()['theta']
        
        #Optimal case: both goal posts seen:
        if post == self.enemyGoalBoth:
            if leftHeading > 0.0 and rightHeading < 0.0:
                self.initialScanComplete = False
                self.alignFinished = True
            elif leftHeading <= 0.0:
                Util.move(self.SHUFFLE_FORWARD, self.SHUFFLE_LEFT, -self.SHUFFLE_TURN)
            else:
                Util.move(self.SHUFFLE_FORWARD, -self.SHUFFLE_LEFT, self.SHUFFLE_TURN)
                
        #Execute initial scan to determine if robot is facing the middle of the enemy goal
        elif not self.initialScanComplete:
            Util.move(0, 0, 0.0)
            if self.globalHeading > math.pi:
                self.initialScanComplete = True
                self.setDirection()
                self.state = 2
            else:
                Util.headScan(self.waypoints, self.state)
                if self.state == 0 and (post == self.enemyGoalLeft or post == self.enemyGoalEither) and leftHeading > 0.0:
                    self.state = 1 - self.state
                elif self.state == 1 and (post == self.enemyGoalRight or post == self.enemyGoalEither) and leftHeading < 0.0:
                    self.initialScanComplete = False
                    self.alignFinished = True
                else:
                    realPitch = Robot.sensorValues()['joints']['angles'][Joints.HeadPitch]
                    realYaw = Robot.sensorValues()['joints']['angles'][Joints.HeadYaw]
                    if abs(realPitch - Info.head()._pitch) < radians(10) and abs(realYaw - Info.head()._yaw) < radians(10):
                        self.initialScanComplete = True
                        self.setDirection()
                        self.state = 2
         
        #Shuffle to align on the goal
        else:
            Util.headScan(self.waypoints, self.state)
            Util.move(self.SHUFFLE_FORWARD, -self.turnDirection*self.SHUFFLE_LEFT, self.turnDirection*self.SHUFFLE_TURN)
            if (self.turnDirection == 1 and (post == self.enemyGoalRight or post == self.enemyGoalEither) and rightHeading < -0.2) or (self.turnDirection == -1 and (post == self.enemyGoalLeft or post == self.enemyGoalEither) and leftHeading > 0.2):
                self.initialScanComplete = False
                self.alignFinished = True
    
    def setDirection(self):
        if self.globalHeading <= math.pi/2.0 or self.globalHeading > 3.0*math.pi/2.0:
            self.turnDirection = 1
        elif self.globalHeading > math.pi/2.0 and self.globalHeading <= 3.0*math.pi/2.0:
            self.turnDirection = -1

