import ActionCommand
import Joints
import Robot
import WhichCamera
import GameController
import Info
from FSM import Behavior
from math import degrees, radians, atan2, pi

class CheckGoal(Behavior):
    originalPitch = 0.0
    originalYaw = 0.0
    wasUsingTopCamera = False
    targetPitch = 0.0
    targetYaw = 0.0
    

    def setup(self):
        self.firstRunComplete = False
        self.goalLookComplete = False
        self.finished = False
        self.wasUsingTopCamera = False

    def step(self):
        if not self.firstRunComplete:
            self.setupWaypoints()
            self.firstRunComplete = True
        else:
            pitch = Robot.sensorValues()['joints']['angles'][Joints.HeadPitch]
            yaw = Robot.sensorValues()['joints']['angles'][Joints.HeadYaw]
            if abs(yaw - self.targetYaw) > radians(10.0) or abs(pitch - self.targetPitch) > radians(10.0):
                Info.head().yaw(self.targetYaw)
                Info.head().yawSpeed(.3)
                Info.head().pitch(self.targetPitch)
                Info.head().pitchSpeed(.2)
                print "Target: ", self.targetYaw * 180.0 / pi, ", Current: ", yaw
            elif not self.goalLookComplete:
                self.targetYaw = self.originalYaw
                self.targetPitch = self.originalPitch
                if self.wasUsingTopCamera:
                    Robot.setCamera(WhichCamera.TOP_CAMERA)
                else:
                    Robot.setCamera(WhichCamera.BOTTOM_CAMERA)
                self.goalLookComplete = True
            else:
                self.firstRunComplete = False
                self.finished = True
            
    def setupWaypoints(self):
        #Determine current position (position to return to when finished)
        self.originalPitch = Robot.sensorValues()['joints']['angles'][Joints.HeadPitch]
        self.originalYaw = Robot.sensorValues()['joints']['angles'][Joints.HeadYaw]
        if Robot.whichCamera() == WhichCamera.TOP_CAMERA:
            self.wasUsingTopCamera = True
        else:
            self.wasUsingTopCamera = False
            
        x = Robot.position()['x']
        y = Robot.position()['y']
        theta = Robot.position()['theta']
        self.targetPitch = 0.0
        #calculate which goal center is closer to look at
        x1 = x
        y1 = 3000-y
        x2 = x
        y2 = -3000 - y
        beta1 = atan2(y1, x1)
        beta2 = atan2(y2, x2)
        alpha1 = pi - beta1 - theta
        alpha2 = pi - beta2 - theta
        if alpha1 <= pi:
            alpha1 += 2*pi
        elif alpha1 > pi:
            alpha1 -= 2*pi
        if alpha2 <= -pi:
            alpha2 += 2*pi
        elif alpha2 > pi:
            alpha2 -= 2*pi
        alpha = alpha1
        if alpha > alpha2:
            alpha = alpha2
        if alpha > radians(115.0):
            alpha = radians(115.0)
        elif alpha < radians(-115.0):
            alpha = radians(-115.0)
        self.targetYaw = alpha
        Robot.setCamera(WhichCamera.TOP_CAMERA)
        
