import Joints
import Robot
import ActionCommand
import WhichPosts
import WhichCamera
import Info
from FSM import Behavior
from math import degrees, radians, atan2
from time import time

# set up a set of waypoints for the head to track which is
# executed in sequence
class ScanForBallSkill(Behavior):
    BOTTOM = radians(25.0)
    MIDDLE = radians(-3.0)
    TOP = radians(17.0)
    YAW_MAX = radians(60.0)
    TOPMIDDLE = radians(3.0)  
 
    starttime = 0.0
    isTurning = False
 
    S_FEET = 0
    S_LBOTTOM = 1
    S_LMIDDLE = 2
    S_LTOP = 3
    S_RBOTTOM = 4
    S_RMIDDLE = 5
    S_RTOP = 6
    state = 1
   
    def setup(self):
        self.isTurning = False
        self.waypoints = []
        self.waypoints.append((self.BOTTOM, -self.YAW_MAX, 0.3, WhichCamera.BOTTOM_CAMERA))  # bottom left
        self.waypoints.append((self.BOTTOM, self.YAW_MAX, 0.3, WhichCamera.BOTTOM_CAMERA))     # top right
        self.waypoints.append((self.MIDDLE, self.YAW_MAX, 0.3, WhichCamera.BOTTOM_CAMERA)) # middle left
        self.waypoints.append((self.MIDDLE, -self.YAW_MAX, 0.3, WhichCamera.BOTTOM_CAMERA)) # middle right
        self.waypoints.append((self.TOP, -self.YAW_MAX, 0.3, WhichCamera.TOP_CAMERA))     # top left
        self.waypoints.append((self.TOP, self.YAW_MAX, 0.3, WhichCamera.TOP_CAMERA))  # bottom right
        self.waypoints.append((self.TOPMIDDLE, self.YAW_MAX, 0.3, WhichCamera.TOP_CAMERA))  # bottom right
        self.waypoints.append((self.TOPMIDDLE, -self.YAW_MAX, 0.3, WhichCamera.TOP_CAMERA))  # bottom right
        self.waypoints.append((self.TOPMIDDLE, -self.YAW_MAX, 0.3, WhichCamera.TOP_CAMERA))  # bottom right
     # self.waypoints.append((self.BOTTOM, 0, 0.3, WhichCamera.BOTTOM_CAMERA))  # bottom left
     # self.waypoints.append((self.MIDDLE, 0, 0.3, WhichCamera.BOTTOM_CAMERA)) # middle left
     # self.waypoints.append((self.TOP, 0, 0.3, WhichCamera.TOP_CAMERA))     # top left
#      self.waypoints.append((self.TOPMIDDLE, -self.YAW_MAX, 0.3, WhichCamera.TOP_CAMERA))  # bottom right

    def reset(self):
        self.state = self.S_FEET
        self.isTurning = False

    def step(self):
        
        #if Robot.lBallLostCount() <= 15:
         #   return True

        if self.state != self.S_FEET:
            Robot.setCamera(self.waypoints[self.state][3])
 
            Info.head().pitch(self.waypoints[self.state][0])
            Info.head().yaw(self.waypoints[self.state][1] * 1.0)
            Info.head().yawSpeed(self.waypoints[self.state][2])
            Info.head().isRelative(False)
            Info.head().pitchSpeed(.2)

            realPitch = Robot.sensorValues()['joints']['angles'][Joints.HeadPitch]
            realYaw = Robot.sensorValues()['joints']['angles'][Joints.HeadYaw]
      
            if (abs(realPitch - Info.head()._pitch) < radians(10) and abs(realYaw - Info.head()._yaw) < radians(10)):
                #self.starttime = 0
                self.state = (self.state + 1) % len(self.waypoints)

        elif self.state == self.S_FEET:
            Info.body().actionType(ActionCommand.Body.FAST)
            Info.body().turn(0.2) 
            
            if not self.isTurning:
                self.starttime = time()
                self.isTurning = True

            if time() - self.starttime >= 2.5:
                self.isTurning = False
                self.state = (self.state + 1) % len(self.waypoints)

        return True

