
#include <vector>
#include <iostream>
#include "perception/localisation/LocalisationAdapter.hpp"
#include "blackboard/Blackboard.hpp"
#include "gamecontroller/RoboCupGameControlData.hpp"
#include "utils/log.hpp"

using namespace std;

const std::string LocalisationAdapter::name("Localisation");

LocalisationAdapter::LocalisationAdapter() {
	gIdent = new GoalIdent();
   //This starts as a particle filter and switches to a Kalman later on
   //rLocaliser = new ParticleFilter();
   rLocaliser = new KalmanFilter();
   AbsCoord startPos;
   //startPos.x = -2300.0;
   //startPos.y = 1650.0;
   //startPos.theta = 0.0;
   startPos.x = 0.0;
   startPos.y = 0.0;
   startPos.theta = 3.14159/2.0;
   startPos.var[0] = 0.0;
   startPos.var[1] = 0.0;
   startPos.var[2] = 0.0;
   std::vector<AbsCoord> startVec;
   startVec.push_back(startPos);
   rLocaliser->robotPos = startVec;
   rLocaliser->numRobotPos = 1;
   writeTo(localisation, numRobotPos, rLocaliser->numRobotPos);
   writeTo(localisation, robotPos, rLocaliser->robotPos);
}

LocalisationAdapter::~LocalisationAdapter() {
}

void LocalisationAdapter::tick() {
	//TODO make this switch localizer systems when conditions met
    llog(DEBUG1) << "Localisation.. ticking away" << endl;

    RoboCupGameControlData stateData;
    stateData = readFrom(gameController, data);
    if (stateData.state == STATE_SET) 
    {
        rLocaliser->robotPos[0].x = 0.0;
        rLocaliser->robotPos[0].y = 0.0;
        rLocaliser->robotPos[0].theta = 3.14159/2.0;
    }

    // update robot localisation
    populateLocaliser(rLocaliser);
    //gIdent->identifyGoal();
    rLocaliser->localise();
    writeTo(localisation, numRobotPos, rLocaliser->numRobotPos);
    writeTo(localisation, robotPos, rLocaliser->robotPos);
    if (rLocaliser->robotPos[0].var[0] + rLocaliser->robotPos[0].var[1] + rLocaliser->robotPos[0].var[2] > 3.0)
    {
        writeTo(localisation, robotLost, 1);
    }
    else
    {
        writeTo(localisation, robotLost, 0);
    }
    
    // update ball tracking
    B.obsNumBalls = readFrom(vision, numBalls);
    if (B.obsNumBalls > 0) {
      B.obsBall = readFrom(vision, ball[0]);
    }
    B.update();
    writeTo(vision, numBalls, B.obsNumBalls);
    writeTo(localisation, ballLostCount, B.lostCount);
}

void LocalisationAdapter::populateLocaliser(Localiser *L) { 
    int i;
    
   // clear all data arrays
   memset(L->obsPosts, 0, sizeof(RRCoord) * MAX_POSTS);
   // set number of robot positions to 1 (possibly just remove this variable later)
   L->numRobotPos = 1;
   // get data from blackboard
   const std::vector<AbsCoord> &oldPos = readFrom(localisation, robotPos);
   if (oldPos.size() > 0) {
      L->robotPos = oldPos;
   } else {
      AbsCoord startPos;
      startPos.x = -230.0;
      startPos.y = 165.0;
      startPos.theta = 0.0;
      startPos.var[0] = 0.0;
      startPos.var[1] = 0.0;
      startPos.var[2] = 0.0;
      std::vector<AbsCoord> startVec;
      startVec.push_back(startPos);
      L->robotPos = startVec;
   }
   
   // update obstacle tracking
    L->obsNumRobots = readFrom(vision, numRobots);
    //cout << "Robots: " << readFrom(vision, numRobots) << endl;
    for (i = 0; i < L->obsNumRobots; i++)
    {
        L->obsRobotTypes[i] = readFrom(vision, robotTypes)[i];
        L->obsRobotLocations[i] = readFrom(vision, robotLocations)[i];
        //cout << "Read: Distance of: " << readFrom(vision, robotLocations)[i].distance << endl;
    }
    
   L->acActive = readFrom(motion, active);
   L->odometry = readFrom(motion, odometry);
   L->obsWhichPosts = readFrom(vision, posts);
   L->obsTeamRed = readFrom(gameController, team_red);
   readArray(vision, post, L->obsPosts);
   L->numSeenEdges = readFrom(vision, numEdges);
   readArray(vision, edges, L->seenEdges);
   L->player_number = readFrom(gameController, player_number);
   L->our_team = readFrom(gameController, our_team);
}
