
#include <vector>
#include "perception/localisation/LocalisationAdapter.hpp"
#include "blackboard/Blackboard.hpp"
#include "utils/log.hpp"

using namespace std;

const std::string LocalisationAdapter::name("Localisation");

LocalisationAdapter::LocalisationAdapter() {
   rLocaliser = new RobotLocaliser();
}

LocalisationAdapter::~LocalisationAdapter() {
}

void LocalisationAdapter::tick() {
    llog(DEBUG1) << "Localisation.. ticking away" << endl;

    // update robot localisation
    populateLocaliser(rLocaliser);
    rLocaliser->localise();
    writeTo(localisation, numRobotPos, rLocaliser->numRobotPos);
    writeTo(localisation, robotPos, rLocaliser->robotPos);

    // update ball tracking
    B.obsNumBalls = readFrom(vision, numBalls);
    B.update();
    writeTo(localisation, ballLostCount, B.lostCount);
}

void LocalisationAdapter::populateLocaliser(Localiser *L) {
   // clear all data arrays
   memset(L->obsPosts, 0, sizeof(RRCoord) * MAX_POSTS);
   // set number of robot positions to 1 (possibly just remove this variable later)
   L->numRobotPos = 1;
   // get data from blackboard
   const std::vector<AbsCoord> &oldPos = readFrom(localisation, robotPos);
   if (oldPos.size() > 0) {
      L->robotPos = oldPos;
   } else {
      AbsCoord startPos;
      startPos.x = 0;
      startPos.y = 0;
      startPos.theta = 0.0;
      startPos.var[0] = 0.0;
      startPos.var[1] = 0.0;
      startPos.var[2] = 0.0;
      std::vector<AbsCoord> startVec;
      startVec.push_back(startPos);
      L->robotPos = startVec;
   }
   L->acActive = readFrom(motion, active);
   L->odometry = readFrom(motion, odometry);
   L->obsWhichPosts = readFrom(vision, posts);
   readArray(vision, post, L->obsPosts);
}
