#include <malloc.h>
#include <limits.h>
#include <stdio.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <fstream>
#include <ctime>
#include <iostream>
#include "perception/vision/NaoV4Camera.hpp"
#include "utils/log.hpp"
#include "utils/speech.hpp"
#include "utils/Timer.hpp"

using namespace std;
using namespace AL;

/**
 * Set the initial settings for the camera(format, etc.)
 */
NaoV4Camera::NaoV4Camera()
{
  // open sempahore
  semaphore1 = sem_open(CAMERA_SEMAPHORE1, O_RDWR);
  if (semaphore1 < 0)
    throw std::runtime_error("NaoV4Camera: sem_open() failed");
  
  // open sempahore
  semaphore2 = sem_open(CAMERA_SEMAPHORE2, O_RDWR);
  if (semaphore2 < 0)
    throw std::runtime_error("NaoV4Camera: sem_open() failed");

  // open shared memory as RW
  shared_fd = shm_open(CAMERA_MEMORY, O_RDWR, 0600);
  if (shared_fd < 0)
    throw std::runtime_error("NaoV4Camera: shm_open() failed");

  // map shared memory to process memory
  shared_data = (CameraData*) mmap(NULL, sizeof(CameraData),
                                   PROT_READ | PROT_WRITE,
                                   MAP_SHARED, shared_fd, 0);
  if (shared_data == MAP_FAILED)
    throw std::runtime_error("NaoV4Camera: mmap() failed");

  /*if (pthread_mutex_trylock(&shared_data->lock)) {
    llog(QUIET) << "NaoV4Camera: Another warriors is already running!" << std::endl;
    std::cerr << "NaoV4Camera: Another warriors is already running!" << std::endl;
    exit(1);
  }*/
  
  dumpFile = NULL;

  llog(INFO) << "NaoV4Camera constructed" << std::endl;
}

NaoV4Camera::~NaoV4Camera()
{
  llog(INFO) << "NaoV4Camera destroying" << std::endl;
  //pthread_mutex_unlock(&shared_data->lock);
  if (shared_data != MAP_FAILED) munmap(shared_data, sizeof(CameraData));
  if (shared_fd >= 0) close(shared_fd);
  if (semaphore1 != SEM_FAILED) sem_close(semaphore1);
  if (semaphore2 != SEM_FAILED) sem_close(semaphore2);
  llog(INFO) << "NaoV4Camera destroyed" << std::endl;
}

const uint8_t *NaoV4Camera::get(const int colourSpace)
{
  if (colourSpace != kYUVColorSpace)
    throw runtime_error("only yuv422 is supported!");

  //cout << "waiting 2" << endl;
  sem_wait(semaphore2);
  //cout << "waiting 1" << endl;
  sem_wait(semaphore1);
  shared_data->readable = !shared_data->readable;
  const uint8_t *image = shared_data->image_data[shared_data->readable];
  //cout << "Got image" << endl;
  sem_post(semaphore1);
   
  writeFrame(image);
  llog(DEBUG2) << "image returning from NaoV4Camera: " << (void *)image << endl;
  return image;
}

bool NaoV4Camera::setCamera( WhichCamera whichCamera )
{
  if (whichCamera == getCamera())
    return true;
    
  shared_data->switchCamera = (int) whichCamera;
  cout << "Changed camera to: " << (int) whichCamera << endl;

  return true;
}

WhichCamera NaoV4Camera::getCamera( )
{
  return (WhichCamera) shared_data->currentCamera;
}

bool NaoV4Camera::setControl( const uint32_t id, const int32_t value )
{
  return true;
}


