#pragma once

#include <alvision/alvisiondefinitions.h>
#include <string>
#include <semaphore.h>
#include "libcamera/CameraData.hpp"

#include "perception/vision/RobotCamera.hpp"

/**
 * NaoCamera provides methods for the vision module to interact with the nao camera
 */
class NaoV4Camera : public RobotCamera {
  public:

   NaoV4Camera();

   virtual ~NaoV4Camera();

   /**
    * tells the caller the current image dimensions
    *
    * @param width will be set to the image's width
    * @param height will be set to the image's height
    * @return if a correct format has been set
    */
   bool imgDimensions(int *const width, int *const height);

   /**
    * Get the image and convert it to the requested colourSpace (currently only
    * kYUVColorSpace (YUV422) is supported)
    *
    * @return a pointer to the image if the image was fetched successfully.
    * NULL, otherwise.
    */
   const uint8_t *get(const int colourSpace);

   /**
    * If applicable, changes the current camera being used.  If necesary,
    * disables the video stream momentarily so the stream can switch inputs.
    *
    * @param whichCamera specifies the top or bottom camera to switch to
    * @return whether the set succeeds
    * @sideeffect stops capturing, switches the camera, then starts again
    */
   bool setCamera(WhichCamera whichCamera);

   /**
    * Gets the current camera
    *
    * @return either the top or bottom camera
    */
   WhichCamera getCamera();

   /**
    * Sets a camera control (eg. hue)
    *
    * @param control the control id to set.
    * @param value the value to set the control to
    * @returns true if success, false otherwise
    */
   bool setControl(const uint32_t id, const int32_t value);

  private:
   int shared_fd;
   CameraData* shared_data;
   sem_t* semaphore1;
   sem_t* semaphore2;

   WhichCamera currentCamera;
};
